@@ -20,7 +20,7 @@
  * 
  */
 /**
-    File Name:          11.2.2-1.js
+    File Name:          11.2.2-6-n.js
     ECMA Section:       11.2.2. The new operator
     Description:
 
@@ -62,7 +62,7 @@
     Date:               12 november 1997
 */
 
-    var SECTION = "11.2.2-1-n.js";
+    var SECTION = "11.2.2-6-n.js";
     var VERSION = "ECMA_1";
     startTest();
     var TITLE   = "The new operator";
@@ -72,12 +72,12 @@
     var testcases = new Array();
 
 
-    var OBJECT = new Object();
+    var STRING = new String("hi");
 
     testcases[tc++] = new TestCase( SECTION,
-                                    "OBJECT = new Object; var o = new OBJECT()",
+                                    "var STRING = new String('hi'); var s = new STRING()",
                                     "error",
-                                    o = new OBJECT() );
+                                    s = new STRING() );
     test();
 
 function TestFunction() {
