@@ -31,6 +31,7 @@
 /**
  * @constructor
  * @extends {WebInspector.DialogDelegate}
+ * @implements {WebInspector.ViewportControl.Provider}
  * @param {WebInspector.SelectionDialogContentProvider} delegate
  */
 WebInspector.FilteredItemSelectionDialog = function(delegate)
@@ -42,31 +43,27 @@ WebInspector.FilteredItemSelectionDialog = function(delegate)
     xhr.send(null);
 
     this.element = document.createElement("div");
-    this.element.className = "js-outline-dialog";
+    this.element.className = "filtered-item-list-dialog";
     this.element.addEventListener("keydown", this._onKeyDown.bind(this), false);
-    this.element.addEventListener("mousemove", this._onMouseMove.bind(this), false);
-    this.element.addEventListener("click", this._onClick.bind(this), false);
     var styleElement = this.element.createChild("style");
     styleElement.type = "text/css";
     styleElement.textContent = xhr.responseText;
 
-    this._itemElements = [];
-    this._elementIndexes = new Map();
-    this._elementHighlightChanges = new Map();
-
     this._promptElement = this.element.createChild("input", "monospace");
     this._promptElement.type = "text";
     this._promptElement.setAttribute("spellcheck", "false");
 
     this._progressElement = this.element.createChild("div", "progress");
 
-    this._itemElementsContainer = document.createElement("div");
-    this._itemElementsContainer.className = "container monospace";
-    this._itemElementsContainer.addEventListener("scroll", this._onScroll.bind(this), false);
+    this._filteredItems = [];
+    this._viewportControl = new WebInspector.ViewportControl(this);
+    this._itemElementsContainer = this._viewportControl.element;
+    this._itemElementsContainer.addStyleClass("container");
+    this._itemElementsContainer.addStyleClass("monospace");
+    this._itemElementsContainer.addEventListener("click", this._onClick.bind(this), false);
     this.element.appendChild(this._itemElementsContainer);
 
     this._delegate = delegate;
-
     this._delegate.requestItems(this._itemsLoaded.bind(this));
 }
 
@@ -94,6 +91,8 @@ WebInspector.FilteredItemSelectionDialog.prototype = {
     focus: function()
     {
         WebInspector.setCurrentFocusElement(this._promptElement);
+        if (this._filteredItems.length && this._viewportControl.lastVisibleIndex() === -1)
+            this._viewportControl.refresh();
     },
 
     willHide: function()
@@ -101,193 +100,242 @@ WebInspector.FilteredItemSelectionDialog.prototype = {
         if (this._isHiding)
             return;
         this._isHiding = true;
+        this._delegate.dispose();
         if (this._filterTimer)
             clearTimeout(this._filterTimer);
     },
 
+    renderAsTwoRows: function()
+    {
+        this._renderAsTwoRows = true;
+    },
+
     onEnter: function()
     {
-        if (!this._selectedElement)
+        if (!this._delegate.itemsCount())
             return;
-        this._delegate.selectItem(this._elementIndexes.get(this._selectedElement), this._promptElement.value.trim());
+        this._delegate.selectItem(this._filteredItems[this._selectedIndexInFiltered], this._promptElement.value.trim());
     },
 
     /**
-     * @param {number} index
-     * @param {number} chunkLength
-     * @param {number} chunkIndex
-     * @param {number} chunkCount
+     * @param {number} loadedCount
+     * @param {number} totalCount
      */
-    _itemsLoaded: function(index, chunkLength, chunkIndex, chunkCount)
+    _itemsLoaded: function(loadedCount, totalCount)
     {
-        for (var i = index; i < index + chunkLength; ++i)
-            this._itemElementsContainer.appendChild(this._createItemElement(i));
-        this._filterItems();
+        this._loadedCount = loadedCount;
+        this._totalCount = totalCount;
 
-        if (chunkIndex === chunkCount)
+        if (this._loadTimeout)
+            return;
+        this._loadTimeout = setTimeout(this._updateAfterItemsLoaded.bind(this), 100);
+    },
+
+    _updateAfterItemsLoaded: function()
+    {
+        delete this._loadTimeout;
+        this._filterItems();
+        if (this._loadedCount === this._totalCount)
             this._progressElement.style.backgroundImage = "";
         else {
             const color = "rgb(66, 129, 235)";
-            const percent = ((chunkIndex / chunkCount) * 100) + "%";
+            const percent = ((this._loadedCount / this._totalCount) * 100) + "%";
             this._progressElement.style.backgroundImage = "-webkit-linear-gradient(left, " + color + ", " + color + " " + percent + ",  transparent " + percent + ")";
         }
     },
 
     /**
      * @param {number} index
+     * @return {Element}
      */
     _createItemElement: function(index)
     {
-        if (this._itemElements[index])
-            return this._itemElements[index];
-
         var itemElement = document.createElement("div");
-        itemElement.className = "item";
+        itemElement.className = "filtered-item-list-dialog-item " + (this._renderAsTwoRows ? "two-rows" : "one-row");
         itemElement._titleElement = itemElement.createChild("span");
         itemElement._titleElement.textContent = this._delegate.itemTitleAt(index);
         itemElement._titleSuffixElement = itemElement.createChild("span");
-        itemElement._subtitleElement = itemElement.createChild("span", "subtitle");
-        itemElement._subtitleElement.textContent = this._delegate.itemSubtitleAt(index);
-        this._elementIndexes.put(itemElement, index);
-        this._itemElements.push(itemElement);
+        itemElement._titleSuffixElement.textContent = this._delegate.itemSuffixAt(index);
+        itemElement._subtitleElement = itemElement.createChild("div", "filtered-item-list-dialog-subtitle");
+        itemElement._subtitleElement.textContent = this._delegate.itemSubtitleAt(index) || "\u200B";
+        itemElement._index = index;
+
+        var key = this._delegate.itemKeyAt(index);
+        var ranges = [];
+        var match;
+        if (this._query) {
+            var regex = this._createSearchRegex(this._query, true);
+            while ((match = regex.exec(key)) !== null && match[0])
+                ranges.push({ offset: match.index, length: regex.lastIndex - match.index });
+            if (ranges.length)
+                WebInspector.highlightRangesWithStyleClass(itemElement, ranges, "highlight");
+        }
+        if (index === this._filteredItems[this._selectedIndexInFiltered])
+            itemElement.addStyleClass("selected");
+
         return itemElement;
     },
 
-    /**
-     * @param {Element} itemElement
-     */
-    _hideItemElement: function(itemElement)
-    {
-        itemElement.style.display = "none";
-    },
-
-    /**
-     * @param {Element} itemElement
-     */
-    _itemElementVisible: function(itemElement)
-    {
-        return itemElement.style.display !== "none";
-    },
-
-    /**
-     * @param {Element} itemElement
-     */
-    _showItemElement: function(itemElement)
-    {
-        itemElement.style.display = "";
-    },
-
-    /**
-     * @param {string} query
-     * @param {boolean=} isGlobal
-     */
-    _createSearchRegExp: function(query, isGlobal)
-    {
-        return this._innerCreateSearchRegExp(this._delegate.rewriteQuery(query), isGlobal);
-    },
-
     /**
      * @param {?string} query
      * @param {boolean=} isGlobal
      */
-    _innerCreateSearchRegExp: function(query, isGlobal)
+    _createSearchRegex: function(query, isGlobal)
     {
-        if (!query)
-            return new RegExp(".*");
-        query = query.trim();
+        const toEscape = String.regexSpecialCharacters();
+        var regexString = "";
+        for (var i = 0; i < query.length; ++i) {
+            var c = query.charAt(i);
+            if (toEscape.indexOf(c) !== -1)
+                c = "\\" + c;
+            if (i)
+                regexString += "[^" + c + "]*";
+            regexString += c;
+        }
+        return new RegExp(regexString, "i" + (isGlobal ? "g" : ""));
+    },
 
-        var ignoreCase = (query === query.toLowerCase());
-        var regExpString = query.escapeForRegExp().replace(/\\\*/g, ".*").replace(/\\\?/g, ".")
-        if (ignoreCase)
-            regExpString = regExpString.replace(/(?!^)(\\\.|[_:-])/g, "[^._:-]*$1");
-        else
-            regExpString = regExpString.replace(/(?!^)(\\\.|[A-Z_:-])/g, "[^.A-Z_:-]*$1");
-        regExpString = regExpString;
-        return new RegExp(regExpString, (ignoreCase ? "i" : "") + (isGlobal ? "g" : ""));
+    /**
+     * @param {string} query
+     * @param {boolean} ignoreCase
+     * @param {boolean} camelCase
+     * @return {RegExp}
+     */
+    _createScoringRegex: function(query, ignoreCase, camelCase)
+    {
+        if (!camelCase || (camelCase && ignoreCase))
+            query = query.toUpperCase();
+        var regexString = "";
+        for (var i = 0; i < query.length; ++i) {
+            var c = query.charAt(i);
+            if (c < "A" || c > "Z")
+               continue;
+            if (regexString)
+               regexString += camelCase ? "[^A-Z]*" : "[^-_ .]*[-_ .]";
+            regexString += c;
+        }
+        if (!camelCase)
+            regexString = "(?:^|[-_ .])" + regexString;
+        return new RegExp(regexString, camelCase ? "" : "i");
+    },
+
+    /**
+     * @param {string} query
+     */
+    setQuery: function(query)
+    {
+        this._promptElement.value = query;
+        this._scheduleFilter();
     },
 
     _filterItems: function()
     {
         delete this._filterTimer;
 
-        var query = this._promptElement.value;
-        query = query.trim();
-        var regex = this._createSearchRegExp(query);
+        var query = this._delegate.rewriteQuery(this._promptElement.value.trim());
+        this._query = query;
 
-        var firstElement;
-        for (var i = 0; i < this._itemElements.length; ++i) {
-            var itemElement = this._itemElements[i];
-            itemElement._titleSuffixElement.textContent = this._delegate.itemSuffixAt(i);
-            if (regex.test(this._delegate.itemKeyAt(i))) {
-                this._showItemElement(itemElement);
-                if (!firstElement)
-                    firstElement = itemElement;
-            } else
-                this._hideItemElement(itemElement);
+        var ignoreCase = (query === query.toLowerCase());
+
+        var filterRegex = query ? this._createSearchRegex(query) : null;
+        var camelCaseScoringRegex = query ? this._createScoringRegex(query, ignoreCase, true) : null;
+        var underscoreScoringRegex = query ? this._createScoringRegex(query, ignoreCase, false) : null;
+
+        var oldSelectedAbsoluteIndex = this._selectedIndexInFiltered ? this._filteredItems[this._selectedIndexInFiltered] : null;
+        this._filteredItems = [];
+        this._selectedIndexInFiltered = 0;
+
+        var cachedKeys = new Array(this._delegate.itemsCount());
+        var scores = query ? new Array(this._delegate.itemsCount()) : null;
+
+        for (var i = 0; i < this._delegate.itemsCount(); ++i) {
+            var key = this._delegate.itemKeyAt(i);
+            if (filterRegex && !filterRegex.test(key))
+                continue;
+            cachedKeys[i] = key;
+            this._filteredItems.push(i);
+
+            if (!filterRegex)
+                continue;
+
+            var score = 0;
+            if (underscoreScoringRegex.test(key))
+                score += 10;
+            if (camelCaseScoringRegex.test(key))
+                score += ignoreCase ? 10 : 20;
+            for (var j = 0; j < key.length && j < query.length; ++j) {
+                if (key[j] === query[j])
+                    score++;
+                if (key[j].toUpperCase() === query[j].toUpperCase())
+                    score++;
+                else
+                    break;
+            }
+            scores[i] = score;
         }
 
-        if (!this._selectedElement || !this._itemElementVisible(this._selectedElement))
-            this._updateSelection(firstElement);
-
-        if (query) {
-            this._highlightItems(query);
-            this._query = query;
-        } else {
-            this._clearHighlight();
-            delete this._query;
+        function compareFunction(index1, index2)
+        {
+            if (scores) {
+                var score1 = scores[index1];
+                var score2 = scores[index2];
+                if (score1 > score2)
+                    return -1;
+                if (score1 < score2)
+                    return 1;
+            }
+            var key1 = cachedKeys[index1];
+            var key2 = cachedKeys[index2];
+            return key1.compareTo(key2) || (index2 - index1);
         }
+
+        const numberOfItemsToSort = 100;
+        if (this._filteredItems.length > numberOfItemsToSort)
+            this._filteredItems.sortRange(compareFunction.bind(this), 0, this._filteredItems.length - 1, numberOfItemsToSort);
+        else
+            this._filteredItems.sort(compareFunction.bind(this));
+
+        for (var i = 0; i < this._filteredItems.length; ++i) {
+            if (this._filteredItems[i] === oldSelectedAbsoluteIndex) {
+                this._selectedIndexInFiltered = i;
+                break;
+            }
+        }
+        this._viewportControl.refresh();
+        this._updateSelection(this._selectedIndexInFiltered, false);
     },
 
     _onKeyDown: function(event)
     {
-        function nextItem(itemElement, isPageScroll, forward)
-        {
-            var scrollItemsLeft = isPageScroll && this._rowsPerViewport ? this._rowsPerViewport : 1;
-            var candidate = itemElement;
-            var lastVisibleCandidate = candidate;
-            do {
-                candidate = forward ? candidate.nextSibling : candidate.previousSibling;
-                if (!candidate) {
-                    if (isPageScroll)
-                        return lastVisibleCandidate;
-                    else
-                        candidate = forward ? this._itemElementsContainer.firstChild : this._itemElementsContainer.lastChild;
-                }
-                if (!this._itemElementVisible(candidate))
-                    continue;
-                lastVisibleCandidate = candidate;
-                --scrollItemsLeft;
-            } while (scrollItemsLeft && candidate !== this._selectedElement);
+        var newSelectedIndex = this._selectedIndexInFiltered;
 
-            return candidate;
+        switch (event.keyCode) {
+        case WebInspector.KeyboardShortcut.Keys.Down.code:
+            if (++newSelectedIndex >= this._filteredItems.length)
+                newSelectedIndex = this._filteredItems.length - 1;
+            this._updateSelection(newSelectedIndex, true);
+            event.consume(true);
+            break;
+        case WebInspector.KeyboardShortcut.Keys.Up.code:
+            if (--newSelectedIndex < 0)
+                newSelectedIndex = 0;
+            this._updateSelection(newSelectedIndex, false);
+            event.consume(true);
+            break;
+        case WebInspector.KeyboardShortcut.Keys.PageDown.code:
+            newSelectedIndex = Math.min(newSelectedIndex + this._viewportControl.rowsPerViewport(), this._filteredItems.length - 1);
+            this._updateSelection(newSelectedIndex, true);
+            event.consume(true);
+            break;
+        case WebInspector.KeyboardShortcut.Keys.PageUp.code:
+            newSelectedIndex = Math.max(newSelectedIndex - this._viewportControl.rowsPerViewport(), 0);
+            this._updateSelection(newSelectedIndex, false);
+            event.consume(true);
+            break;
+        default:
+            if (event.keyIdentifier !== "Shift" && event.keyIdentifier !== "Ctrl" && event.keyIdentifier !== "Meta" && event.keyIdentifier !== "Left" && event.keyIdentifier !== "Right")
+                this._scheduleFilter();
         }
-
-        if (this._selectedElement) {
-            var candidate;
-            switch (event.keyCode) {
-            case WebInspector.KeyboardShortcut.Keys.Down.code:
-                candidate = nextItem.call(this, this._selectedElement, false, true);
-                break;
-            case WebInspector.KeyboardShortcut.Keys.Up.code:
-                candidate = nextItem.call(this, this._selectedElement, false, false);
-                break;
-            case WebInspector.KeyboardShortcut.Keys.PageDown.code:
-                candidate = nextItem.call(this, this._selectedElement, true, true);
-                break;
-            case WebInspector.KeyboardShortcut.Keys.PageUp.code:
-                candidate = nextItem.call(this, this._selectedElement, true, false);
-                break;
-            }
-
-            if (candidate) {
-                this._updateSelection(candidate);
-                event.preventDefault();
-                return;
-            }
-        }
-
-        if (event.keyIdentifier !== "Shift" && event.keyIdentifier !== "Ctrl" && event.keyIdentifier !== "Meta" && event.keyIdentifier !== "Left" && event.keyIdentifier !== "Right")
-            this._scheduleFilter();
     },
 
     _scheduleFilter: function()
@@ -298,117 +346,48 @@ WebInspector.FilteredItemSelectionDialog.prototype = {
     },
 
     /**
-     * @param {Element} newSelectedElement
+     * @param {number} index  
+     * @param {boolean} makeLast
      */
-    _updateSelection: function(newSelectedElement)
-    {
-        if (this._selectedElement === newSelectedElement)
-            return;
-        if (this._selectedElement)
-            this._selectedElement.removeStyleClass("selected");
-
-        this._selectedElement = newSelectedElement;
-        if (newSelectedElement) {
-            newSelectedElement.addStyleClass("selected");
-            newSelectedElement.scrollIntoViewIfNeeded(false);
-            if (!this._itemHeight) {
-                this._itemHeight = newSelectedElement.offsetHeight;
-                this._rowsPerViewport = Math.floor(this._itemElementsContainer.offsetHeight / this._itemHeight);
-            }
-        }
+    _updateSelection: function(index, makeLast)
+    { 
+        var element = this._viewportControl.renderedElementAt(this._selectedIndexInFiltered);
+        if (element)
+            element.removeStyleClass("selected");
+        this._viewportControl.scrollItemIntoView(index, makeLast);
+        this._selectedIndexInFiltered = index;
+        element = this._viewportControl.renderedElementAt(index);
+        if (element)
+            element.addStyleClass("selected");
     },
 
     _onClick: function(event)
     {
-        var itemElement = event.target.enclosingNodeOrSelfWithClass("item");
+        var itemElement = event.target.enclosingNodeOrSelfWithClass("filtered-item-list-dialog-item");
         if (!itemElement)
             return;
-        this._updateSelection(itemElement);
-        this._delegate.selectItem(this._elementIndexes.get(this._selectedElement), this._promptElement.value.trim());
+        this._delegate.selectItem(itemElement._index, this._promptElement.value.trim());
         WebInspector.Dialog.hide();
     },
 
-    _onMouseMove: function(event)
+    /**
+     * @return {number}
+     */
+    itemCount: function()
     {
-        var itemElement = event.target.enclosingNodeOrSelfWithClass("item");
-        if (!itemElement)
-            return;
-        this._updateSelection(itemElement);
-    },
-
-    _onScroll: function()
-    {
-        if (this._query)
-            this._highlightItems(this._query);
-        else
-            this._clearHighlight();
+        return this._filteredItems.length;
     },
 
     /**
-     * @param {string} query
+     * @param {number} index
+     * @return {Element}
      */
-    _highlightItems: function(query)
+    itemElement: function(index)
     {
-        var regex = this._createSearchRegExp(query, true);
-        for (var i = 0; i < this._delegate.itemsCount(); ++i) {
-            var itemElement = this._itemElements[i];
-            if (this._itemElementVisible(itemElement) && this._itemElementInViewport(itemElement))
-                this._highlightItem(itemElement, regex);
-        }
-    },
-
-    _clearHighlight: function()
-    {
-        for (var i = 0; i < this._delegate.itemsCount(); ++i)
-            this._clearElementHighlight(this._itemElements[i]);
-    },
-
-    /**
-     * @param {Element} itemElement
-     */
-    _clearElementHighlight: function(itemElement)
-    {
-        var changes = this._elementHighlightChanges.get(itemElement)
-        if (changes) {
-            WebInspector.revertDomChanges(changes);
-            this._elementHighlightChanges.remove(itemElement);
-        }
-    },
-
-    /**
-     * @param {Element} itemElement
-     * @param {RegExp} regex
-     */
-    _highlightItem: function(itemElement, regex)
-    {
-        this._clearElementHighlight(itemElement);
-
-        var key = this._delegate.itemKeyAt(this._elementIndexes.get(itemElement));
-        var ranges = [];
-
-        var match;
-        while ((match = regex.exec(key)) !== null && match[0]) {
-            ranges.push({ offset: match.index, length: regex.lastIndex - match.index });
-        }
-
-        var changes = [];
-        WebInspector.highlightRangesWithStyleClass(itemElement, ranges, "highlight", changes);
-
-        if (changes.length)
-            this._elementHighlightChanges.put(itemElement, changes);
-    },
-
-    /**
-     * @param {Element} itemElement
-     * @return {boolean}
-     */
-    _itemElementInViewport: function(itemElement)
-    {
-        if (itemElement.offsetTop + this._itemHeight < this._itemElementsContainer.scrollTop)
-            return false;
-        if (itemElement.offsetTop > this._itemElementsContainer.scrollTop + this._itemHeight * (this._rowsPerViewport + 1))
-            return false;
-        return true;
+        var delegateIndex = this._filteredItems[index];
+        var element = this._createItemElement(delegateIndex);
+        element._filteredIndex = index;
+        return element;
     },
 
     __proto__: WebInspector.DialogDelegate.prototype
@@ -452,7 +431,7 @@ WebInspector.SelectionDialogContentProvider.prototype = {
     itemsCount: function() { },
 
     /**
-     * @param {function(number, number, number, number)} callback
+     * @param {function(number, number)} callback
      */
     requestItems: function(callback) { },
 
@@ -467,6 +446,8 @@ WebInspector.SelectionDialogContentProvider.prototype = {
      * @return {string}
      */
     rewriteQuery: function(query) { },
+
+    dispose: function() { }
 }
 
 /**
@@ -544,7 +525,7 @@ WebInspector.JavaScriptOutlineDialog.prototype = {
     },
 
     /**
-     * @param {function(number, number, number, number)} callback
+     * @param {function(number, number)} callback
      */
     requestItems: function(callback)
     {
@@ -568,12 +549,10 @@ WebInspector.JavaScriptOutlineDialog.prototype = {
     _didBuildOutlineChunk: function(callback, event)
     {
         var data = event.data;
-
-        var index = this._functionItems.length;
         var chunk = data["chunk"];
         for (var i = 0; i < chunk.length; ++i)
             this._functionItems.push(chunk[i]);
-        callback(index, chunk.length, data.index, data.total);
+        callback(data.index, data.total);
 
         if (data.total === data.index && this._outlineWorker) {
             this._outlineWorker.terminate();
@@ -602,43 +581,58 @@ WebInspector.JavaScriptOutlineDialog.prototype = {
         return query;
     },
 
-    __proto__: WebInspector.SelectionDialogContentProvider.prototype
+    dispose: function()
+    {
+    }
 }
 
 /**
  * @constructor
  * @implements {WebInspector.SelectionDialogContentProvider}
- * @param {WebInspector.ScriptsPanel} panel
- * @param {WebInspector.UISourceCodeProvider} uiSourceCodeProvider
  */
-WebInspector.OpenResourceDialog = function(panel, uiSourceCodeProvider)
+WebInspector.SelectUISourceCodeDialog = function()
 {
-    WebInspector.SelectionDialogContentProvider.call(this);
-    this._panel = panel;
-
-    this._uiSourceCodes = uiSourceCodeProvider.uiSourceCodes();
-
-    function filterOutEmptyURLs(uiSourceCode)
-    {
-        return !!uiSourceCode.parsedURL.lastPathComponent;
-    }
-    this._uiSourceCodes = this._uiSourceCodes.filter(filterOutEmptyURLs);
-
-    function compareFunction(uiSourceCode1, uiSourceCode2)
-    {
-        return uiSourceCode1.parsedURL.lastPathComponent.localeCompare(uiSourceCode2.parsedURL.lastPathComponent);
-    }
-    this._uiSourceCodes.sort(compareFunction);
+    var projects = WebInspector.workspace.projects().filter(this.filterProject.bind(this));
+    this._uiSourceCodes = [];
+    for (var i = 0; i < projects.length; ++i)
+        this._uiSourceCodes = this._uiSourceCodes.concat(projects[i].uiSourceCodes().filter(this.filterUISourceCode.bind(this)));
+    WebInspector.workspace.addEventListener(WebInspector.UISourceCodeProvider.Events.UISourceCodeAdded, this._uiSourceCodeAdded, this);
 }
 
-WebInspector.OpenResourceDialog.prototype = {
+WebInspector.SelectUISourceCodeDialog.prototype = {
+    /**
+     * @param {WebInspector.UISourceCode} uiSourceCode
+     * @param {number} lineNumber
+     */
+    uiSourceCodeSelected: function(uiSourceCode, lineNumber)
+    {
+        // Overridden by subclasses
+    },
+
+    /**
+     * @param {WebInspector.Project} project
+     */
+    filterProject: function(project)
+    {
+        return true;
+        // Overridden by subclasses
+    },
+
+    /**
+     * @param {WebInspector.UISourceCode} uiSourceCode
+     */
+    filterUISourceCode: function(uiSourceCode)
+    {
+        return uiSourceCode.name();
+    },
+
     /**
      * @param {number} itemIndex
      * @return {string}
      */
     itemTitleAt: function(itemIndex)
     {
-        return this._uiSourceCodes[itemIndex].parsedURL.lastPathComponent;
+        return this._uiSourceCodes[itemIndex].name().trimEnd(100);
     },
 
     /*
@@ -656,7 +650,12 @@ WebInspector.OpenResourceDialog.prototype = {
      */
     itemSubtitleAt: function(itemIndex)
     {
-        return this._uiSourceCodes[itemIndex].parsedURL.folderPathComponents;
+        var uiSourceCode = this._uiSourceCodes[itemIndex]
+        var projectName = uiSourceCode.project().displayName();
+        var path = uiSourceCode.path().slice();
+        path.pop();
+        path.unshift(projectName);
+        return path.join("/");
     },
 
     /**
@@ -665,7 +664,7 @@ WebInspector.OpenResourceDialog.prototype = {
      */
     itemKeyAt: function(itemIndex)
     {
-        return this._uiSourceCodes[itemIndex].parsedURL.lastPathComponent;
+        return this._uiSourceCodes[itemIndex].name();
     },
 
     /**
@@ -677,11 +676,12 @@ WebInspector.OpenResourceDialog.prototype = {
     },
 
     /**
-     * @param {function(number, number, number, number)} callback
+     * @param {function(number, number)} callback
      */
     requestItems: function(callback)
     {
-        callback(0, this._uiSourceCodes.length, 1, 1);
+        this._itemsLoaded = callback;
+        this._itemsLoaded(1, 1);
     },
 
     /**
@@ -692,7 +692,7 @@ WebInspector.OpenResourceDialog.prototype = {
     {
         var lineNumberMatch = promptValue.match(/[^:]+\:([\d]*)$/);
         var lineNumber = lineNumberMatch ? Math.max(parseInt(lineNumberMatch[1], 10) - 1, 0) : 0;
-        this._panel.showUISourceCode(this._uiSourceCodes[itemIndex], lineNumber);
+        this.uiSourceCodeSelected(this._uiSourceCodes[itemIndex], lineNumber);
     },
 
     /**
@@ -709,19 +709,117 @@ WebInspector.OpenResourceDialog.prototype = {
         return lineNumberMatch ? lineNumberMatch[1] : query;
     },
 
-    __proto__: WebInspector.SelectionDialogContentProvider.prototype
+    /**
+     * @param {WebInspector.Event} event
+     */
+    _uiSourceCodeAdded: function(event)
+    {
+        var uiSourceCode = /** @type {WebInspector.UISourceCode} */ (event.data);
+        if (!this.filterUISourceCode(uiSourceCode))
+            return;
+        this._uiSourceCodes.push(uiSourceCode)
+        this._itemsLoaded(1, 1);
+    },
+
+    dispose: function()
+    {
+        WebInspector.workspace.removeEventListener(WebInspector.UISourceCodeProvider.Events.UISourceCodeAdded, this._uiSourceCodeAdded, this);
+    }
+}
+
+/**
+ * @constructor
+ * @extends {WebInspector.SelectUISourceCodeDialog}
+ * @param {WebInspector.ScriptsPanel} panel
+ */
+WebInspector.OpenResourceDialog = function(panel)
+{
+    WebInspector.SelectUISourceCodeDialog.call(this);
+    this._panel = panel;
+}
+
+WebInspector.OpenResourceDialog.prototype = {
+
+    /**
+     * @param {WebInspector.UISourceCode} uiSourceCode
+     * @param {number} lineNumber
+     */
+    uiSourceCodeSelected: function(uiSourceCode, lineNumber)
+    {
+        this._panel.showUISourceCode(uiSourceCode, lineNumber);
+    },
+
+    /**
+     * @param {WebInspector.Project} project
+     */
+    filterProject: function(project)
+    {
+        return !project.isServiceProject();
+    },
+
+    __proto__: WebInspector.SelectUISourceCodeDialog.prototype
 }
 
 /**
  * @param {WebInspector.ScriptsPanel} panel
- * @param {WebInspector.UISourceCodeProvider} uiSourceCodeProvider
  * @param {Element} relativeToElement
  */
-WebInspector.OpenResourceDialog.show = function(panel, uiSourceCodeProvider, relativeToElement)
+WebInspector.OpenResourceDialog.show = function(panel, relativeToElement)
 {
     if (WebInspector.Dialog.currentInstance())
         return;
 
-    var filteredItemSelectionDialog = new WebInspector.FilteredItemSelectionDialog(new WebInspector.OpenResourceDialog(panel, uiSourceCodeProvider));
+    var filteredItemSelectionDialog = new WebInspector.FilteredItemSelectionDialog(new WebInspector.OpenResourceDialog(panel));
+    filteredItemSelectionDialog.renderAsTwoRows();
+    WebInspector.Dialog.show(relativeToElement, filteredItemSelectionDialog);
+}
+
+/**
+ * @constructor
+ * @extends {WebInspector.SelectUISourceCodeDialog}
+ * @param {string} type
+ * @param {function(WebInspector.UISourceCode)} callback
+ */
+WebInspector.SelectUISourceCodeForProjectTypeDialog = function(type, callback)
+{
+    this._type = type;
+    WebInspector.SelectUISourceCodeDialog.call(this);
+    this._callback = callback;
+}
+
+WebInspector.SelectUISourceCodeForProjectTypeDialog.prototype = {
+    /**
+     * @param {WebInspector.UISourceCode} uiSourceCode
+     * @param {number} lineNumber
+     */
+    uiSourceCodeSelected: function(uiSourceCode, lineNumber)
+    {
+        this._callback(uiSourceCode);
+    },
+
+    /**
+     * @param {WebInspector.Project} project
+     */
+    filterProject: function(project)
+    {
+        return project.type() === this._type;
+    },
+
+    __proto__: WebInspector.SelectUISourceCodeDialog.prototype
+}
+
+/**
+ * @param {string} type
+ * @param {function(WebInspector.UISourceCode)} callback
+ * @param {Element} relativeToElement
+ */
+WebInspector.SelectUISourceCodeForProjectTypeDialog.show = function(name, type, callback, relativeToElement)
+{
+    if (WebInspector.Dialog.currentInstance())
+        return;
+
+    var filteredItemSelectionDialog = new WebInspector.FilteredItemSelectionDialog(new WebInspector.SelectUISourceCodeForProjectTypeDialog(type, callback));
+    filteredItemSelectionDialog.setQuery(name);
+    filteredItemSelectionDialog.renderAsTwoRows();
     WebInspector.Dialog.show(relativeToElement, filteredItemSelectionDialog);
 }
