@@ -28,9 +28,9 @@
  * @constructor
  * @extends {WebInspector.PropertiesSection}
  * @param {WebInspector.RemoteObject} object
- * @param {string|Element=} title
+ * @param {?string|Element=} title
  * @param {string=} subtitle
- * @param {string=} emptyPlaceholder
+ * @param {?string=} emptyPlaceholder
  * @param {boolean=} ignoreHasOwnProperty
  * @param {Array.<WebInspector.RemoteObjectProperty>=} extraProperties
  * @param {function(new:TreeElement, WebInspector.RemoteObjectProperty)=} treeElementConstructor
@@ -84,7 +84,7 @@ WebInspector.ObjectPropertiesSection.prototype = {
         {
             if (!properties)
                 return;
-            this.updateProperties(properties);
+            this.updateProperties(properties, internalProperties);
         }
 
         if (this.ignoreHasOwnProperty)
@@ -93,7 +93,7 @@ WebInspector.ObjectPropertiesSection.prototype = {
             this.object.getOwnProperties(callback.bind(this));
     },
 
-    updateProperties: function(properties, rootTreeElementConstructor, rootPropertyComparer)
+    updateProperties: function(properties, internalProperties, rootTreeElementConstructor, rootPropertyComparer)
     {
         if (!rootTreeElementConstructor)
             rootTreeElementConstructor = this.treeElementConstructor;
@@ -101,25 +101,20 @@ WebInspector.ObjectPropertiesSection.prototype = {
         if (!rootPropertyComparer)
             rootPropertyComparer = WebInspector.ObjectPropertiesSection.CompareProperties;
 
-        if (this.extraProperties)
+        if (this.extraProperties) {
             for (var i = 0; i < this.extraProperties.length; ++i)
                 properties.push(this.extraProperties[i]);
-
-        properties.sort(rootPropertyComparer);
+        }
 
         this.propertiesTreeOutline.removeChildren();
 
-        for (var i = 0; i < properties.length; ++i) {
-            if (this.skipProto && properties[i].name === "__proto__")
-                continue;
-            properties[i].parentObject = this.object;
-        }
-
+        WebInspector.ObjectPropertyTreeElement.populateWithProperties(this.propertiesTreeOutline,
+            properties, internalProperties,
+            rootTreeElementConstructor, rootPropertyComparer,
+            this.skipProto, this.object);
+            
         this.propertiesForTest = properties;
 
-        for (var i = 0; i < properties.length; ++i)
-            this.propertiesTreeOutline.appendChild(new rootTreeElementConstructor(properties[i]));
-
         if (!this.propertiesTreeOutline.children.length) {
             var title = document.createElement("div");
             title.className = "info";
@@ -245,7 +240,7 @@ WebInspector.ObjectPropertyTreeElement.prototype = {
             this.valueElement.addStyleClass("console-formatted-" + this.property.value.type);
 
         this.valueElement.addEventListener("contextmenu", this._contextMenuFired.bind(this, this.property.value), false);
-        if (this.property.value.type === "object" && this.property.value.subtype === "node") {
+        if (this.property.value.type === "object" && this.property.value.subtype === "node" && this.property.value.description) {
             WebInspector.DOMPresentationUtils.createSpansForNodeTitle(this.valueElement, this.property.value.description);
             this.valueElement.addEventListener("mousemove", this._mouseMove.bind(this, this.property.value), false);
             this.valueElement.addEventListener("mouseout", this._mouseOut.bind(this, this.property.value), false);
@@ -456,42 +451,60 @@ WebInspector.ObjectPropertyTreeElement.populate = function(treeElement, value) {
         treeElement.removeChildren();
         if (!properties)
             return;
+        if (!internalProperties)
+            internalProperties = [];
 
-        properties.sort(WebInspector.ObjectPropertiesSection.CompareProperties);
-        for (var i = 0; i < properties.length; ++i) {
-            if (treeElement.treeOutline.section.skipProto && properties[i].name === "__proto__")
-                continue;
-            properties[i].parentObject = value;
-            treeElement.appendChild(new treeElement.treeOutline.section.treeElementConstructor(properties[i]));
-        }
-        if (value.type === "function") {
-            // Whether function has TargetFunction internal property.
-            // This is a simple way to tell that the function is actually a bound function (we are not told).
-            // Bound function never has inner scope and doesn't need corresponding UI node.   
-            var hasTargetFunction = false;
-
-            if (internalProperties) {
-                for (var i = 0; i < internalProperties.length; i++) {
-                    if (internalProperties[i].name == "[[TargetFunction]]") {
-                        hasTargetFunction = true;
-                        break;
-                    }
-                }
-            }
-            if (!hasTargetFunction)
-                treeElement.appendChild(new WebInspector.FunctionScopeMainTreeElement(value));
-        }
-        if (internalProperties) {
-            for (var i = 0; i < internalProperties.length; i++) {
-                internalProperties[i].parentObject = value;
-                treeElement.appendChild(new treeElement.treeOutline.section.treeElementConstructor(internalProperties[i]));
-            } 
-        }
+        WebInspector.ObjectPropertyTreeElement.populateWithProperties(treeElement, properties, internalProperties,
+            treeElement.treeOutline.section.treeElementConstructor, WebInspector.ObjectPropertiesSection.CompareProperties,
+            treeElement.treeOutline.section.skipProto, value);
     }
 
     value.getOwnProperties(callback);
 }
 
+/**
+ * @param {!TreeElement|!TreeOutline} treeElement
+ * @param {Array.<!WebInspector.RemoteObjectProperty>} properties
+ * @param {?Array.<!WebInspector.RemoteObjectProperty>} internalProperties
+ * @param {function(new:TreeElement, WebInspector.RemoteObjectProperty)} treeElementConstructor
+ * @param {function (WebInspector.RemoteObjectProperty, WebInspector.RemoteObjectProperty): number} comparator
+ * @param {boolean} skipProto
+ * @param {?WebInspector.RemoteObject} value
+ */
+WebInspector.ObjectPropertyTreeElement.populateWithProperties = function(treeElement, properties, internalProperties, treeElementConstructor, comparator, skipProto, value) {
+    properties.sort(comparator);
+    
+    for (var i = 0; i < properties.length; ++i) {
+        if (skipProto && properties[i].name === "__proto__")
+            continue;
+        properties[i].parentObject = value;
+        treeElement.appendChild(new treeElementConstructor(properties[i]));
+    }
+    if (value && value.type === "function") {
+        // Whether function has TargetFunction internal property.
+        // This is a simple way to tell that the function is actually a bound function (we are not told).
+        // Bound function never has inner scope and doesn't need corresponding UI node.   
+        var hasTargetFunction = false;
+
+        if (internalProperties) {
+            for (var i = 0; i < internalProperties.length; i++) {
+                if (internalProperties[i].name == "[[TargetFunction]]") {
+                    hasTargetFunction = true;
+                    break;
+                }
+            }
+        }
+        if (!hasTargetFunction)
+            treeElement.appendChild(new WebInspector.FunctionScopeMainTreeElement(value));
+    }
+    if (internalProperties) {
+        for (var i = 0; i < internalProperties.length; i++) {
+            internalProperties[i].parentObject = value;
+            treeElement.appendChild(new treeElementConstructor(internalProperties[i]));
+        } 
+    }
+}
+
 /**
  * @constructor
  * @extends {TreeElement}
@@ -550,9 +563,18 @@ WebInspector.FunctionScopeMainTreeElement.prototype = {
                         title = WebInspector.UIString("Global");
                         isTrueObject = true;
                         break;
+                    default:
+                        console.error("Unknown scope type: " + scope.type);
+                        continue;
                 }
+                
+                var scopeRef;
+                if (isTrueObject)
+                    scopeRef = undefined;
+                else 
+                    scopeRef = new WebInspector.ScopeRef(i, undefined, this._remoteObject.objectId);
 
-                var remoteObject = WebInspector.RemoteObject.fromPayload(scope.object);
+                var remoteObject = WebInspector.RemoteObject.fromScopePayload(scope.object, scopeRef);
                 if (isTrueObject) {
                     var property = WebInspector.RemoteObjectProperty.fromScopeValue(title, remoteObject);
                     property.parentObject = null;
@@ -614,6 +636,7 @@ WebInspector.ArrayGroupingTreeElement = function(object, fromIndex, toIndex, pro
 }
 
 WebInspector.ArrayGroupingTreeElement._bucketThreshold = 100;
+WebInspector.ArrayGroupingTreeElement._sparseIterationThreshold = 250000;
 
 /**
  * @param {TreeElement|TreeOutline} treeElement
@@ -635,22 +658,43 @@ WebInspector.ArrayGroupingTreeElement._populateArray = function(treeElement, obj
  */
 WebInspector.ArrayGroupingTreeElement._populateRanges = function(treeElement, object, fromIndex, toIndex, topLevel)
 {
-    object.callFunctionJSON(packRanges, [{value: fromIndex}, {value: toIndex}, {value: WebInspector.ArrayGroupingTreeElement._bucketThreshold}], callback.bind(this));
+    object.callFunctionJSON(packRanges, [{value: fromIndex}, {value: toIndex}, {value: WebInspector.ArrayGroupingTreeElement._bucketThreshold}, {value: WebInspector.ArrayGroupingTreeElement._sparseIterationThreshold}], callback.bind(this));
 
     /**
      * @this {Object}
      * @param {number=} fromIndex // must declare optional
      * @param {number=} toIndex // must declare optional
      * @param {number=} bucketThreshold // must declare optional
+     * @param {number=} sparseIterationThreshold // must declare optional
      */
-    function packRanges(fromIndex, toIndex, bucketThreshold)
+    function packRanges(fromIndex, toIndex, bucketThreshold, sparseIterationThreshold)
     {
-        var count = 0;
-        for (var i = fromIndex; i <= toIndex; ++i) {
-            if (i in this)
-                ++count;
+        var ownPropertyNames = null;
+        function doLoop(iterationCallback)
+        {
+            if (toIndex - fromIndex < sparseIterationThreshold) {
+                for (var i = fromIndex; i <= toIndex; ++i) {
+                    if (i in this)
+                        iterationCallback(i);
+                }
+            } else {
+                ownPropertyNames = ownPropertyNames || Object.getOwnPropertyNames(this);
+                for (var i = 0; i < ownPropertyNames.length; ++i) {
+                    var name = ownPropertyNames[i];
+                    var index = name >>> 0;
+                    if (String(index) === name && fromIndex <= index && index <= toIndex)
+                        iterationCallback(index);
+                }
+            }
         }
 
+        var count = 0;
+        function countIterationCallback()
+        {
+            ++count;
+        }
+        doLoop.call(this, countIterationCallback);
+
         var bucketSize = count;
         if (count <= bucketThreshold)
             bucketSize = count;
@@ -661,10 +705,8 @@ WebInspector.ArrayGroupingTreeElement._populateRanges = function(treeElement, ob
         count = 0;
         var groupStart = -1;
         var groupEnd = 0;
-        for (var i = fromIndex; i <= toIndex; ++i) {
-            if (!(i in this))
-                continue;
-
+        function loopIterationCallback(i)
+        {
             if (groupStart === -1)
                 groupStart = i;
 
@@ -675,6 +717,7 @@ WebInspector.ArrayGroupingTreeElement._populateRanges = function(treeElement, ob
                 groupStart = -1;
             }
         }
+        doLoop.call(this, loopIterationCallback);
 
         if (count > 0)
             ranges.push([groupStart, groupEnd, count]);
@@ -709,23 +752,35 @@ WebInspector.ArrayGroupingTreeElement._populateRanges = function(treeElement, ob
  */
 WebInspector.ArrayGroupingTreeElement._populateAsFragment = function(treeElement, object, fromIndex, toIndex)
 {
-    object.callFunction(buildArrayFragment, [{value: fromIndex}, {value: toIndex}], processArrayFragment.bind(this));
+    object.callFunction(buildArrayFragment, [{value: fromIndex}, {value: toIndex}, {value: WebInspector.ArrayGroupingTreeElement._sparseIterationThreshold}], processArrayFragment.bind(this));
 
     /**
      * @this {Object}
      * @param {number=} fromIndex // must declare optional
      * @param {number=} toIndex // must declare optional
+     * @param {number=} sparseIterationThreshold // must declare optional
      */
-    function buildArrayFragment(fromIndex, toIndex)
+    function buildArrayFragment(fromIndex, toIndex, sparseIterationThreshold)
     {
         var result = Object.create(null);
-        for (var i = fromIndex; i <= toIndex; ++i) {
-            if (i in this)
-                result[i] = this[i];
+        if (toIndex - fromIndex < sparseIterationThreshold) {
+            for (var i = fromIndex; i <= toIndex; ++i) {
+                if (i in this)
+                    result[i] = this[i];
+            }
+        } else {
+            var ownPropertyNames = Object.getOwnPropertyNames(this);
+            for (var i = 0; i < ownPropertyNames.length; ++i) {
+                var name = ownPropertyNames[i];
+                var index = name >>> 0;
+                if (String(index) === name && fromIndex <= index && index <= toIndex)
+                    result[index] = this[index];
+            }
         }
         return result;
     }
 
+    /** @this {WebInspector.ArrayGroupingTreeElement} */
     function processArrayFragment(arrayFragment)
     {
         arrayFragment.getAllProperties(processProperties.bind(this));
@@ -762,7 +817,8 @@ WebInspector.ArrayGroupingTreeElement._populateNonIndexProperties = function(tre
         var names = Object.getOwnPropertyNames(this);
         for (var i = 0; i < names.length; ++i) {
             var name = names[i];
-            if (!isNaN(name))
+            // Array index check according to the ES5-15.4.
+            if (String(name >>> 0) === name && name >>> 0 !== 0xffffffff)
                 continue;
             var descriptor = Object.getOwnPropertyDescriptor(this, name);
             if (descriptor)
