@@ -134,12 +134,12 @@ void SincResampler::consumeSource(float* buffer, unsigned numberOfSourceFrames)
         return;
     
     // Wrap the provided buffer by an AudioBus for use by the source provider.
-    AudioBus bus(1, numberOfSourceFrames, false);
+    RefPtr<AudioBus> bus = AudioBus::create(1, numberOfSourceFrames, false);
 
     // FIXME: Find a way to make the following const-correct:
-    bus.setChannelMemory(0, buffer, numberOfSourceFrames);
+    bus->setChannelMemory(0, buffer, numberOfSourceFrames);
     
-    m_sourceProvider->provideInput(&bus, numberOfSourceFrames);
+    m_sourceProvider->provideInput(bus.get(), numberOfSourceFrames);
 }
 
 namespace {
