@@ -59,6 +59,7 @@ QT_BEGIN_NAMESPACE
 class QPainter;
 class QRect;
 class QOpenGLContext;
+class QOpenGLExtensions;
 class QSurface;
 QT_END_NAMESPACE
 #elif PLATFORM(GTK) || PLATFORM(EFL)
@@ -84,14 +85,9 @@ const Platform3DObject NullPlatform3DObject = 0;
 namespace WebCore {
 class DrawingBuffer;
 class Extensions3D;
-#if USE(OPENGL_ES_2)
+class Extensions3DOpenGLCommon;
 class Extensions3DOpenGLES;
-#else
 class Extensions3DOpenGL;
-#endif
-#if PLATFORM(QT)
-class Extensions3DQt;
-#endif
 class HostWindow;
 class Image;
 class ImageBuffer;
@@ -1013,16 +1009,11 @@ private:
     String mappedSymbolName(Platform3DObject program, ANGLEShaderSymbolType, const String& name);
     String originalSymbolName(Platform3DObject program, ANGLEShaderSymbolType, const String& name);
 
-    ANGLEWebKitBridge m_compiler;
 #endif
 
-#if PLATFORM(BLACKBERRY) || (PLATFORM(QT) && defined(QT_OPENGL_ES_2)) || ((PLATFORM(GTK) || PLATFORM(EFL) || PLATFORM(WIN)) && USE(OPENGL_ES_2))
-    friend class Extensions3DOpenGLES;
-    OwnPtr<Extensions3DOpenGLES> m_extensions;
-#else
+    OwnPtr<Extensions3DOpenGLCommon> m_extensions;
     friend class Extensions3DOpenGL;
-    OwnPtr<Extensions3DOpenGL> m_extensions;
-#endif
+    friend class Extensions3DOpenGLES;
     friend class Extensions3DOpenGLCommon;
 
     Attributes m_attrs;
@@ -1066,6 +1057,10 @@ private:
     // Errors raised by synthesizeGLError().
     ListHashSet<GC3Denum> m_syntheticErrors;
 
+#if PLATFORM(QT)
+    QOpenGLExtensions* m_functions;
+#endif
+
 #if PLATFORM(BLACKBERRY)
     bool m_isImaginationHardware;
 #endif
@@ -1074,6 +1069,7 @@ private:
     friend class GraphicsContext3DPrivate;
     OwnPtr<GraphicsContext3DPrivate> m_private;
 #endif
+    ANGLEWebKitBridge m_compiler;
 };
 
 } // namespace WebCore
