@@ -22,7 +22,7 @@
 #define DISABLE_SHIMS
 #include "OpenGLShims.h"
 
-#if !PLATFORM(QT)
+#if !PLATFORM(QT) && !PLATFORM(WIN)
 #include <dlfcn.h>
 #endif
 
@@ -40,7 +40,14 @@ OpenGLFunctionTable* openGLFunctionTable()
 #if PLATFORM(QT)
 static void* getProcAddress(const char* procName)
 {
-    return reinterpret_cast<void*>(QOpenGLContext::currentContext()->getProcAddress(procName));
+    if (QOpenGLContext* context = QOpenGLContext::currentContext())
+        return reinterpret_cast<void*>(context->getProcAddress(procName));
+    return 0;
+}
+#elif PLATFORM(WIN)
+static void* getProcAddress(const char* procName)
+{
+    return GetProcAddress(GetModuleHandleA("libGLESv2"), procName);
 }
 #else
 typedef void* (*glGetProcAddressType) (const char* procName);
@@ -84,6 +91,8 @@ static void* lookupOpenGLFunctionAddress(const char* functionName, bool* success
     fullFunctionName = functionName;
     fullFunctionName.append("ANGLE");
     target = getProcAddress(fullFunctionName.utf8().data());
+    if (target)
+        return target;
 
     fullFunctionName = functionName;
     fullFunctionName.append("APPLE");
@@ -127,6 +136,7 @@ bool initializeOpenGLShims()
     ASSIGN_FUNCTION_TABLE_ENTRY(glBindBuffer, success);
     ASSIGN_FUNCTION_TABLE_ENTRY(glBindFramebuffer, success);
     ASSIGN_FUNCTION_TABLE_ENTRY(glBindRenderbuffer, success);
+    ASSIGN_FUNCTION_TABLE_ENTRY_EXT(glBindVertexArray);
     ASSIGN_FUNCTION_TABLE_ENTRY(glBlendColor, success);
     ASSIGN_FUNCTION_TABLE_ENTRY(glBlendEquation, success);
     ASSIGN_FUNCTION_TABLE_ENTRY(glBlendEquationSeparate, success);
@@ -150,6 +160,7 @@ bool initializeOpenGLShims()
     ASSIGN_FUNCTION_TABLE_ENTRY(glDeleteProgram, success);
     ASSIGN_FUNCTION_TABLE_ENTRY(glDeleteRenderbuffers, success);
     ASSIGN_FUNCTION_TABLE_ENTRY(glDeleteShader, success);
+    ASSIGN_FUNCTION_TABLE_ENTRY_EXT(glDeleteVertexArrays);
     ASSIGN_FUNCTION_TABLE_ENTRY(glDetachShader, success);
     ASSIGN_FUNCTION_TABLE_ENTRY(glDisableVertexAttribArray, success);
     ASSIGN_FUNCTION_TABLE_ENTRY(glEnableVertexAttribArray, success);
@@ -159,6 +170,7 @@ bool initializeOpenGLShims()
     ASSIGN_FUNCTION_TABLE_ENTRY(glGenerateMipmap, success);
     ASSIGN_FUNCTION_TABLE_ENTRY(glGenFramebuffers, success);
     ASSIGN_FUNCTION_TABLE_ENTRY(glGenRenderbuffers, success);
+    ASSIGN_FUNCTION_TABLE_ENTRY_EXT(glGenVertexArrays);
     ASSIGN_FUNCTION_TABLE_ENTRY(glGetActiveAttrib, success);
     ASSIGN_FUNCTION_TABLE_ENTRY(glGetActiveUniform, success);
     ASSIGN_FUNCTION_TABLE_ENTRY(glGetAttachedShaders, success);
@@ -182,6 +194,7 @@ bool initializeOpenGLShims()
     ASSIGN_FUNCTION_TABLE_ENTRY(glIsProgram, success);
     ASSIGN_FUNCTION_TABLE_ENTRY(glIsRenderbuffer, success);
     ASSIGN_FUNCTION_TABLE_ENTRY(glIsShader, success);
+    ASSIGN_FUNCTION_TABLE_ENTRY_EXT(glIsVertexArray);
     ASSIGN_FUNCTION_TABLE_ENTRY(glLinkProgram, success);
     ASSIGN_FUNCTION_TABLE_ENTRY(glRenderbufferStorage, success);
     // In GLES2 there are optional ANGLE and APPLE extensions for glRenderbufferStorageMultisample.
