@@ -1,5 +1,5 @@
 /*
-    Copyright (C) 2008, 2012 Digia Plc. and/or its subsidiary(-ies)
+    Copyright (C) 2015 The Qt Company Ltd
     Copyright (C) 2007 Staikos Computing Services Inc.  <info@staikos.net>
     Copyright (C) 2008 Holger Hans Peter Freyther
 
@@ -50,10 +50,11 @@ FormDataIODevice::FormDataIODevice(FormData* data)
     , m_currentDelta(0)
     , m_fileSize(0)
     , m_dataSize(0)
+    , m_formData(data)
 {
     setOpenMode(FormDataIODevice::ReadOnly);
 
-    prepareFormElements(data);
+    prepareFormElements();
     prepareCurrentElement();
     computeSize();
 }
@@ -63,19 +64,29 @@ FormDataIODevice::~FormDataIODevice()
     delete m_currentFile;
 }
 
-void FormDataIODevice::prepareFormElements(FormData* formData)
+bool FormDataIODevice::reset()
 {
-    if (!formData)
+    if (m_currentFile)
+        m_currentFile->close();
+
+    m_currentDelta = 0;
+    m_formElements = m_formData->elements();
+
+    prepareCurrentElement();
+    return true;
+}
+
+void FormDataIODevice::prepareFormElements()
+{
+    if (!m_formData)
         return;
 
-    RefPtr<FormData> formDataRef(formData);
-
 #if ENABLE(BLOB)
-    formDataRef = formDataRef->resolveBlobReferences();
+    m_formData = m_formData->resolveBlobReferences();
 #endif
 
     // Take a deep copy of the FormDataElements
-    m_formElements = formDataRef->elements();
+    m_formElements = m_formData->elements();
 }
 
 
@@ -672,6 +683,10 @@ void QNetworkReplyHandler::forwardData()
 {
     ASSERT(m_replyWrapper && m_replyWrapper->reply() && !wasAborted() && !m_replyWrapper->wasRedirected());
 
+    // reply may be closed if reply->close() or reply->abort() is called
+    if (!m_replyWrapper->reply()->isReadable())
+        return;
+
     ResourceHandleClient* client = m_resourceHandle->client();
     if (!client)
         return;
