@@ -42,6 +42,7 @@
  * inside a GTK application.  
  */
 
+#include "config.h"
 #include "GtkVersioning.h"
 #include "xembed.h"
 #include "gtk2xtbin.h"
@@ -273,9 +274,7 @@ gtk_xtbin_realize (GtkWidget *widget)
 {
   GtkXtBin     *xtbin;
   GtkAllocation allocation = { 0, 0, 200, 200 };
-#if GTK_CHECK_VERSION(2, 18, 0)
   GtkAllocation widget_allocation;
-#endif
 
 #ifdef DEBUG_XTBIN
   printf("gtk_xtbin_realize()\n");
@@ -286,29 +285,17 @@ gtk_xtbin_realize (GtkWidget *widget)
   xtbin = GTK_XTBIN (widget);
 
   /* caculate the allocation before realize */
-#if GTK_CHECK_VERSION(2, 24, 0)
   allocation.width = gdk_window_get_width(xtbin->parent_window);
   allocation.height = gdk_window_get_height(xtbin->parent_window);
-#else
-  gint  x, y, w, h, d; /* geometry of window */
-  gdk_window_get_geometry(xtbin->parent_window, &x, &y, &w, &h, &d);
-  allocation.width = w;
-  allocation.height = h;
-#endif
   gtk_widget_size_allocate (widget, &allocation);
 
 #ifdef DEBUG_XTBIN
   printf("initial allocation %d %d %d %d\n", x, y, w, h);
 #endif
 
-#if GTK_CHECK_VERSION(2, 18, 0)
   gtk_widget_get_allocation(widget, &widget_allocation);
   xtbin->width = widget_allocation.width;
   xtbin->height = widget_allocation.height;
-#else
-  xtbin->width = widget->allocation.width;
-  xtbin->height = widget->allocation.height;
-#endif
 
   /* use GtkSocket's realize */
   (*GTK_WIDGET_CLASS(parent_class)->realize)(widget);
