@@ -345,6 +345,17 @@ static bool isWindowsMessageUserGesture(UINT message)
     }
 }
 
+static inline bool isWebViewVisible(FrameView* view)
+{
+#if PLATFORM(QT)
+    if (PlatformPageClient client = view->hostWindow()->platformPageClient())
+        return client->isViewVisible();
+    return false;
+#else
+    return true;
+#endif // PLATFORM(QT)
+}
+
 static inline IntPoint contentsToNativeWindow(FrameView* view, const IntPoint& point)
 {
 #if PLATFORM(QT)
@@ -454,7 +465,6 @@ void PluginView::updatePluginWidget()
 #endif
     m_clipRect = windowClipRect();
     m_clipRect.move(-m_windowRect.x(), -m_windowRect.y());
-
     if (platformPluginWidget() && (!m_haveUpdatedPluginWidget || m_windowRect != oldWindowRect || m_clipRect != oldClipRect)) {
         HRGN rgn;
 
@@ -473,8 +483,10 @@ void PluginView::updatePluginWidget()
             ::SetWindowRgn(platformPluginWidget(), rgn, TRUE);
         }
 
-        if (!m_haveUpdatedPluginWidget || m_windowRect != oldWindowRect)
-            ::MoveWindow(platformPluginWidget(), m_windowRect.x(), m_windowRect.y(), m_windowRect.width(), m_windowRect.height(), TRUE);
+        if (!m_haveUpdatedPluginWidget || m_windowRect != oldWindowRect) {
+            IntRect nativeWindowRect = contentsToNativeWindow(frameView, frameRect());
+            ::MoveWindow(platformPluginWidget(), nativeWindowRect.x(), nativeWindowRect.y(), nativeWindowRect.width(), nativeWindowRect.height(), TRUE);
+        }
 
         if (clipToZeroRect) {
             rgn = ::CreateRectRgn(m_clipRect.x(), m_clipRect.y(), m_clipRect.maxX(), m_clipRect.maxY());
@@ -510,9 +522,10 @@ void PluginView::show()
 {
     setSelfVisible(true);
 
-    if (isParentVisible() && platformPluginWidget())
+    if (isParentVisible() && platformPluginWidget()) {
         ShowWindow(platformPluginWidget(), SW_SHOWNA);
-
+        forceRedraw();
+    }
     Widget::show();
 }
 
@@ -818,9 +831,10 @@ void PluginView::setParentVisible(bool visible)
     Widget::setParentVisible(visible);
 
     if (isSelfVisible() && platformPluginWidget()) {
-        if (visible)
+        if (visible) {
             ShowWindow(platformPluginWidget(), SW_SHOWNA);
-        else
+            forceRedraw();
+        } else
             ShowWindow(platformPluginWidget(), SW_HIDE);
     }
 }
@@ -864,6 +878,8 @@ void PluginView::setNPWindowRect(const IntRect& rect)
         JSC::JSLock::DropAllLocks dropAllLocks(JSDOMWindowBase::commonVM());
         setCallingPlugin(true);
         m_plugin->pluginFuncs()->setwindow(m_instance, &m_npWindow);
+        if (m_plugin->quirks().contains(PluginQuirkNeedsSetWindowTwice))
+            m_plugin->pluginFuncs()->setwindow(m_instance, &m_npWindow);
         setCallingPlugin(false);
 
         m_haveCalledSetWindow = true;
@@ -1016,7 +1032,7 @@ bool PluginView::platformStart()
 #endif
 
         DWORD flags = WS_CHILD;
-        if (isSelfVisible())
+        if (isSelfVisible() && isWebViewVisible(toFrameView(parent())))
             flags |= WS_VISIBLE;
 
         HWND parentWindowHandle = windowHandleForPageClient(m_parentFrame->view()->hostWindow()->platformPageClient());
