@@ -25,7 +25,7 @@
 
 namespace WebCore {
 
-class SVGFEDistantLightElement : public SVGFELightElement {
+class SVGFEDistantLightElement FINAL : public SVGFELightElement {
 public:
     static PassRefPtr<SVGFEDistantLightElement> create(const QualifiedName&, Document*);
 
