@@ -44,6 +44,7 @@
 #import <Foundation/NSURLResponse.h>
 #import <WebCore/Document.h>
 #import <WebCore/DocumentLoader.h>
+#import <WebCore/Editor.h>
 #import <WebCore/Frame.h>
 #import <WebCore/FrameLoader.h>
 #import <WebCore/FrameLoaderClient.h>
@@ -54,6 +55,7 @@
 #import <WebCore/HTMLNames.h>
 #import <WebCore/HTMLTableCellElement.h>
 #import <WebCore/MIMETypeRegistry.h>
+#import <WebCore/NodeTraversal.h>
 #import <WebCore/Range.h>
 #import <WebCore/RegularExpression.h>
 #import <WebCore/RenderObject.h>
@@ -82,44 +84,46 @@ using namespace HTMLNames;
 
 @implementation WebHTMLRepresentation
 
-static NSArray *stringArray(const HashSet<String>& set)
+static NSMutableArray *newArrayWithStrings(const HashSet<String>& set) NS_RETURNS_RETAINED;
+static NSMutableArray *newArrayWithStrings(const HashSet<String>& set)
 {
-    NSMutableArray *array = [NSMutableArray arrayWithCapacity:set.size()];
+    NSMutableArray *array = [[NSMutableArray alloc] initWithCapacity:set.size()];
     HashSet<String>::const_iterator end = set.end();
     for (HashSet<String>::const_iterator it = set.begin(); it != end; ++it)
         [array addObject:(NSString *)(*it)];
     return array;
 }
 
-static NSArray *concatenateArrays(NSArray *first, NSArray *second)
+static NSMutableArray *newArrayByConcatenatingArrays(NSArray *first, NSArray *second) NS_RETURNS_RETAINED;
+static NSMutableArray *newArrayByConcatenatingArrays(NSArray *first, NSArray *second)
 {
-    NSMutableArray *result = [[first mutableCopy] autorelease];
+    NSMutableArray *result = [first mutableCopy];
     [result addObjectsFromArray:second];
     return result;
 }
 
 + (NSArray *)supportedMIMETypes
 {
-    DEFINE_STATIC_LOCAL(RetainPtr<NSArray>, staticSupportedMIMETypes, (concatenateArrays([self supportedNonImageMIMETypes], [self supportedImageMIMETypes])));
-    return staticSupportedMIMETypes.get();
+    static __unsafe_unretained NSArray *staticSupportedMIMETypes = newArrayByConcatenatingArrays([self supportedNonImageMIMETypes], [self supportedImageMIMETypes]);
+    return staticSupportedMIMETypes;
 }
 
 + (NSArray *)supportedNonImageMIMETypes
 {
-    DEFINE_STATIC_LOCAL(RetainPtr<NSArray>, staticSupportedNonImageMIMETypes, (stringArray(MIMETypeRegistry::getSupportedNonImageMIMETypes())));
-    return staticSupportedNonImageMIMETypes.get();
+    static __unsafe_unretained NSArray *staticSupportedNonImageMIMETypes = newArrayWithStrings(MIMETypeRegistry::getSupportedNonImageMIMETypes());
+    return staticSupportedNonImageMIMETypes;
 }
 
 + (NSArray *)supportedImageMIMETypes
 {
-    DEFINE_STATIC_LOCAL(RetainPtr<NSArray>, staticSupportedImageMIMETypes, (stringArray(MIMETypeRegistry::getSupportedImageMIMETypes())));
-    return staticSupportedImageMIMETypes.get();
+    static __unsafe_unretained NSArray *staticSupportedImageMIMETypes = newArrayWithStrings(MIMETypeRegistry::getSupportedImageMIMETypes());
+    return staticSupportedImageMIMETypes;
 }
 
 + (NSArray *)unsupportedTextMIMETypes
 {
-    DEFINE_STATIC_LOCAL(RetainPtr<NSArray>, staticUnsupportedTextMIMETypes, (stringArray(MIMETypeRegistry::getUnsupportedTextMIMETypes())));
-    return staticUnsupportedTextMIMETypes.get();
+    static __unsafe_unretained NSArray *staticUnsupportedTextMIMETypes = newArrayWithStrings(MIMETypeRegistry::getUnsupportedTextMIMETypes());
+    return staticUnsupportedTextMIMETypes;
 }
 
 - (id)init
@@ -216,7 +220,7 @@ static NSArray *concatenateArrays(NSArray *first, NSArray *second)
         return;
     WebView *webView = [webFrame webView];
     if ([webView mainFrame] == webFrame && [webView isEditable])
-        core(webFrame)->editor()->applyEditingStyleToBodyElement();
+        core(webFrame)->editor().applyEditingStyleToBodyElement();
 }
 
 - (BOOL)canProvideDocumentSource
@@ -299,7 +303,7 @@ static HTMLFormElement* formElementFromDOMElement(DOMElement *element)
 static HTMLInputElement* inputElementFromDOMElement(DOMElement* element)
 {
     Element* node = core(element);
-    return node && node->hasTagName(inputTag) ? static_cast<HTMLInputElement*>(node) : 0;
+    return node && isHTMLInputElement(node) ? toHTMLInputElement(node) : 0;
 }
 
 - (BOOL)elementDoesAutoComplete:(DOMElement *)element
@@ -437,12 +441,12 @@ static NSString* searchForLabelsBeforeElement(Frame* frame, NSArray* labels, Ele
     // walk backwards in the node tree, until another element, or form, or end of tree
     unsigned lengthSearched = 0;
     Node* n;
-    for (n = element->traversePreviousNode();
+    for (n = NodeTraversal::previous(element);
          n && lengthSearched < charsSearchedThreshold;
-         n = n->traversePreviousNode())
+         n = NodeTraversal::previous(n))
     {
         if (n->hasTagName(formTag)
-            || (n->isHTMLElement() && static_cast<Element*>(n)->isFormControlElement()))
+            || (n->isHTMLElement() && toElement(n)->isFormControlElement()))
         {
             // We hit another form element or the start of the form - bail out
             break;
