@@ -29,6 +29,7 @@
 #include "qwebscriptworld.h"
 #include "qwebscriptworld_p.h"
 #include "qwebsecurityorigin.h"
+#include "DOMWrapperWorld.h"
 #include <QMultiMap>
 #include <qdebug.h>
 #include <qevent.h>
@@ -209,7 +210,7 @@ QWebFrame::QWebFrame(QWebPage *parentPage)
     d->q = this;
     d->init(/*page adapter*/ parentPage->handle());
 
-#if ENABLE(ORIENTATION_EVENTS)
+#if ENABLE(ORIENTATION_EVENTS) && HAVE(QTSENSORS)
     connect(&d->m_orientation, SIGNAL(readingChanged()), this, SLOT(_q_orientationChanged()));
     d->m_orientation.start();
 #endif
@@ -222,7 +223,7 @@ QWebFrame::QWebFrame(QWebFrame* parent, QWebFrameData* frameData)
     d->page = parent->d->page;
     d->q = this;
     d->init(parent->d->pageAdapter, frameData);
-#if ENABLE(ORIENTATION_EVENTS)
+#if ENABLE(ORIENTATION_EVENTS) && HAVE(QTSENSORS)
     connect(&d->m_orientation, SIGNAL(readingChanged()), this, SLOT(_q_orientationChanged()));
     d->m_orientation.start();
 #endif
@@ -767,9 +768,11 @@ QWebElement QWebFrame::documentElement() const
     Returns a new list of elements matching the given CSS selector \a selectorQuery.
     If there are no matching elements, an empty list is returned.
 
-    \l{http://www.w3.org/TR/REC-CSS2/selector.html#q1}{Standard CSS2 selector} syntax is
+    \l{Standard CSS selector} syntax is
     used for the query.
 
+    This method is equivalent to Document::querySelectorAll in the \l{DOM Selectors API}.
+
     \sa QWebElement::findAll()
 */
 QWebElementCollection QWebFrame::findAllElements(const QString &selectorQuery) const
@@ -783,8 +786,9 @@ QWebElementCollection QWebFrame::findAllElements(const QString &selectorQuery) c
     given CSS selector \a selectorQuery. If there is no matching element, a
     null element is returned.
 
-    \l{http://www.w3.org/TR/REC-CSS2/selector.html#q1}{Standard CSS2 selector} syntax is
-    used for the query.
+    \l{Standard CSS selector} syntax is used for the query.
+
+    This method is equivalent to Document::querySelector in the \l{DOM Selectors API}.
 
     \sa QWebElement::findFirst()
 */
@@ -1160,7 +1164,11 @@ QUrl QWebHitTestResult::linkUrl() const
     return d->linkUrl;
 }
 
+#if QT_DEPRECATED_SINCE(5,5)
 /*!
+    \obsolete
+    Use linkTitleString instead.
+
     Returns the title of the link.
 */
 QUrl QWebHitTestResult::linkTitle() const
@@ -1169,6 +1177,18 @@ QUrl QWebHitTestResult::linkTitle() const
         return QUrl();
     return d->linkTitle;
 }
+#endif // QT_DEPRECATED_SINCE(5,5)
+
+/*!
+    \since 5.5
+    Returns the title of the link.
+*/
+QString QWebHitTestResult::linkTitleString() const
+{
+    if (!d)
+        return QString();
+    return d->linkTitle;
+}
 
 /*!
   \since 4.6
@@ -1215,6 +1235,17 @@ QUrl QWebHitTestResult::imageUrl() const
     return d->imageUrl;
 }
 
+/*!
+    Returns the url of the video or audio element.
+    \since 5.2
+*/
+QUrl QWebHitTestResult::mediaUrl() const
+{
+    if (!d)
+        return QUrl();
+    return d->mediaUrl;
+}
+
 /*!
     Returns a QPixmap containing the image. A null pixmap is returned if the
     element being tested is not an image.
