@@ -1,7 +1,7 @@
 /*
  * This file is part of the theme implementation for form controls in WebCore.
  *
- * Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+ * Copyright (C) 2015 The Qt Company Ltd.
  * Copyright (C) 2011-2012 Nokia Corporation and/or its subsidiary(-ies).
  *
  * This library is free software; you can redistribute it and/or
@@ -248,8 +248,8 @@ void QStyleFacadeImp::paintButton(QPainter* painter, QStyleFacade::ButtonType ty
     QWidget* widget = qobject_cast<QWidget*>(widgetForPainter(painter));
     MappedStyleOption<QStyleOptionButton> option(widget, proxyOption);
 
-    if (m_style->inherits("QWindowsVistaStyle"))
-        option.styleObject = 0;
+    if (option.styleObject)
+        option.styleObject->setProperty("_q_no_animation", true);
 
     if (type == PushButton)
         style()->drawControl(QStyle::CE_PushButton, &option, painter, widget);
