@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # Copyright (C) 2010 Google Inc. All rights reserved.
 # Copyright (C) 2010 Gabor Rapcsanyi (rgabor@inf.u-szeged.hu), University of Szeged
 # Copyright (C) 2011 Apple Inc. All rights reserved.
@@ -29,7 +28,6 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-import errno
 import logging
 import optparse
 import os
@@ -38,10 +36,9 @@ import sys
 import traceback
 
 from webkitpy.common.host import Host
-from webkitpy.common.system import stack_utils
-from webkitpy.layout_tests.controllers.manager import Manager, WorkerException, TestRunInterruptedException
-from webkitpy.layout_tests.models import test_expectations
-from webkitpy.layout_tests.port import configuration_options, platform_options
+from webkitpy.layout_tests.controllers.manager import Manager
+from webkitpy.port import configuration_options, platform_options
+from webkitpy.layout_tests.views import buildbot_results
 from webkitpy.layout_tests.views import printing
 
 
@@ -56,165 +53,52 @@ INTERRUPTED_EXIT_STATUS = signal.SIGINT + 128
 EXCEPTIONAL_EXIT_STATUS = 254
 
 
-def lint(port, options):
-    host = port.host
-    if options.platform:
-        ports_to_lint = [port]
+def main(argv, stdout, stderr):
+    options, args = parse_args(argv)
+
+    if options.platform and 'test' in options.platform:
+        # It's a bit lame to import mocks into real code, but this allows the user
+        # to run tests against the test platform interactively, which is useful for
+        # debugging test failures.
+        from webkitpy.common.host_mock import MockHost
+        host = MockHost()
     else:
-        ports_to_lint = [host.port_factory.get(name) for name in host.port_factory.all_port_names()]
+        host = Host()
 
-    files_linted = set()
-    lint_failed = False
+    if options.lint_test_files:
+        from webkitpy.layout_tests.lint_test_expectations import lint
+        return lint(host, options, stderr)
 
-    for port_to_lint in ports_to_lint:
-        expectations_dict = port_to_lint.expectations_dict()
-        for expectations_file in expectations_dict.keys():
-            if expectations_file in files_linted:
-                continue
-
-            try:
-                test_expectations.TestExpectations(port_to_lint, expectations_to_lint={expectations_file: expectations_dict[expectations_file]})
-            except test_expectations.ParseError, e:
-                lint_failed = True
-                _log.error('')
-                for warning in e.warnings:
-                    _log.error(warning)
-                _log.error('')
-            files_linted.add(expectations_file)
-
-    if lint_failed:
-        _log.error('Lint failed.')
-        return -1
-    _log.info('Lint succeeded.')
-    return 0
-
-
-def run(port, options, args, regular_output=sys.stderr, buildbot_output=sys.stdout):
     try:
-        warnings = _set_up_derived_options(port, options)
+        port = host.port_factory.get(options.platform, options)
+    except NotImplementedError, e:
+        # FIXME: is this the best way to handle unsupported port names?
+        print >> stderr, str(e)
+        return EXCEPTIONAL_EXIT_STATUS
 
-        printer = printing.Printer(port, options, regular_output, buildbot_output, logger=logging.getLogger())
+    try:
+        run_details = run(port, options, args, stderr)
+        if run_details.exit_code != -1:
+            bot_printer = buildbot_results.BuildBotPrinter(stdout, options.debug_rwt_logging)
+            bot_printer.print_results(run_details)
 
-        for warning in warnings:
-            _log.warning(warning)
-
-        if options.lint_test_files:
-            return lint(port, options)
-
-        # We wrap any parts of the run that are slow or likely to raise exceptions
-        # in a try/finally to ensure that we clean up the logging configuration.
-        unexpected_result_count = -1
-
-        manager = Manager(port, options, printer)
-        printer.print_config(port.results_directory())
-
-        unexpected_result_count = manager.run(args)
-        _log.debug("Testing completed, Exit status: %d" % unexpected_result_count)
-    except Exception:
-        exception_type, exception_value, exception_traceback = sys.exc_info()
-        if exception_type not in (KeyboardInterrupt, TestRunInterruptedException, WorkerException):
-            print >> sys.stderr, '\n%s raised: %s' % (exception_type.__name__, exception_value)
-            stack_utils.log_traceback(_log.error, exception_traceback)
-        raise
-    finally:
-        printer.cleanup()
-
-    return unexpected_result_count
+        return run_details.exit_code
+    except KeyboardInterrupt:
+        return INTERRUPTED_EXIT_STATUS
+    except BaseException as e:
+        if isinstance(e, Exception):
+            print >> stderr, '\n%s raised: %s' % (e.__class__.__name__, str(e))
+            traceback.print_exc(file=stderr)
+        return EXCEPTIONAL_EXIT_STATUS
 
 
-def _set_up_derived_options(port, options):
-    """Sets the options values that depend on other options values."""
-    # We return a list of warnings to print after the printer is initialized.
-    warnings = []
-
-    if not options.child_processes:
-        options.child_processes = os.environ.get("WEBKIT_TEST_CHILD_PROCESSES",
-                                                 str(port.default_child_processes()))
-    if not options.max_locked_shards:
-        options.max_locked_shards = int(os.environ.get("WEBKIT_TEST_MAX_LOCKED_SHARDS",
-                                                       str(port.default_max_locked_shards())))
-
-    if not options.configuration:
-        options.configuration = port.default_configuration()
-
-    if options.pixel_tests is None:
-        options.pixel_tests = port.default_pixel_tests()
-
-    if not options.time_out_ms:
-        options.time_out_ms = str(port.default_timeout_ms())
-
-    options.slow_time_out_ms = str(5 * int(options.time_out_ms))
-
-    if options.additional_platform_directory:
-        additional_platform_directories = []
-        for path in options.additional_platform_directory:
-            additional_platform_directories.append(port.host.filesystem.abspath(path))
-        options.additional_platform_directory = additional_platform_directories
-
-    if not options.http and options.skipped in ('ignore', 'only'):
-        warnings.append("--force/--skipped=%s overrides --no-http." % (options.skipped))
-        options.http = True
-
-    if options.ignore_metrics and (options.new_baseline or options.reset_results):
-        warnings.append("--ignore-metrics has no effect with --new-baselines or with --reset-results")
-
-    if options.new_baseline:
-        options.reset_results = True
-        options.add_platform_exceptions = True
-
-    if options.pixel_test_directories:
-        options.pixel_tests = True
-        varified_dirs = set()
-        pixel_test_directories = options.pixel_test_directories
-        for directory in pixel_test_directories:
-            # FIXME: we should support specifying the directories all the ways we support it for additional
-            # arguments specifying which tests and directories to run. We should also move the logic for that
-            # to Port.
-            filesystem = port.host.filesystem
-            if not filesystem.isdir(filesystem.join(port.layout_tests_dir(), directory)):
-                warnings.append("'%s' was passed to --pixel-test-directories, which doesn't seem to be a directory" % str(directory))
-            else:
-                varified_dirs.add(directory)
-
-        options.pixel_test_directories = list(varified_dirs)
-
-    if options.run_singly:
-        options.verbose = True
-
-    return warnings
-
-
-def _compat_shim_callback(option, opt_str, value, parser):
-    print "Ignoring unsupported option: %s" % opt_str
-
-
-def _compat_shim_option(option_name, **kwargs):
-    return optparse.make_option(option_name, action="callback",
-        callback=_compat_shim_callback,
-        help="Ignored, for old-run-webkit-tests compat only.", **kwargs)
-
-
-def parse_args(args=None):
-    """Provides a default set of command line args.
-
-    Returns a tuple of options, args from optparse"""
-
+def parse_args(args):
     option_group_definitions = []
 
     option_group_definitions.append(("Platform options", platform_options()))
     option_group_definitions.append(("Configuration options", configuration_options()))
     option_group_definitions.append(("Printing Options", printing.print_options()))
 
-    # FIXME: These options should move onto the ChromiumPort.
-    option_group_definitions.append(("Chromium-specific Options", [
-        optparse.make_option("--nocheck-sys-deps", action="store_true",
-            default=False,
-            help="Don't check the system dependencies (themes)"),
-        optparse.make_option("--adb-device",
-            action="append", default=[],
-            help="Run Android layout tests on these devices."),
-    ]))
-
     option_group_definitions.append(("EFL-specific Options", [
         optparse.make_option("--webprocess-cmd-prefix", type="string",
             default=False, help="Prefix used when spawning the Web process (Debug mode only)"),
@@ -238,16 +122,10 @@ def parse_args(args=None):
             help="Path to a directory containing the executables needed to run tests."),
     ]))
 
-    option_group_definitions.append(("ORWT Compatibility Options", [
-        # FIXME: Remove this option once the bots don't refer to it.
-        # results.html is smart enough to figure this out itself.
-        _compat_shim_option("--use-remote-links-to-tests"),
-    ]))
-
     option_group_definitions.append(("Results Options", [
-        optparse.make_option("-p", "--pixel-tests", action="store_true",
+        optparse.make_option("-p", "--pixel", "--pixel-tests", action="store_true",
             dest="pixel_tests", help="Enable pixel-to-pixel PNG comparisons"),
-        optparse.make_option("--no-pixel-tests", action="store_false",
+        optparse.make_option("--no-pixel", "--no-pixel-tests", action="store_false",
             dest="pixel_tests", help="Disable pixel-to-pixel PNG comparisons"),
         optparse.make_option("--no-sample-on-timeout", action="store_false",
             dest="sample_on_timeout", help="Don't run sample on timeout (Mac OS X only)"),
@@ -303,18 +181,11 @@ def parse_args(args=None):
             default=True, dest="show_results",
             help="Don't launch a browser with results after the tests "
                  "are done"),
-        # FIXME: We should have a helper function to do this sort of
-        # deprectated mapping and automatically log, etc.
-        optparse.make_option("--noshow-results", action="store_false", dest="show_results", help="Deprecated, same as --no-show-results."),
-        optparse.make_option("--no-launch-safari", action="store_false", dest="show_results", help="Deprecated, same as --no-show-results."),
         optparse.make_option("--full-results-html", action="store_true",
             default=False,
             help="Show all failures in results.html, rather than only regressions"),
         optparse.make_option("--clobber-old-results", action="store_true",
             default=False, help="Clobbers test results from previous runs."),
-        optparse.make_option("--no-record-results", action="store_false",
-            default=True, dest="record_results",
-            help="Don't record the results."),
         optparse.make_option("--http", action="store_true", dest="http",
             default=True, help="Run HTTP and WebSocket tests (default)"),
         optparse.make_option("--no-http", action="store_false", dest="http",
@@ -322,6 +193,12 @@ def parse_args(args=None):
         optparse.make_option("--ignore-metrics", action="store_true", dest="ignore_metrics",
             default=False, help="Ignore rendering metrics related information from test "
             "output, only compare the structure of the rendertree."),
+        optparse.make_option("--nocheck-sys-deps", action="store_true",
+            default=False,
+            help="Don't check the system dependencies (themes)"),
+        optparse.make_option("--nojava", action="store_true",
+            default=False,
+            help="Don't build java support files"),
     ]))
 
     option_group_definitions.append(("Testing Options", [
@@ -391,6 +268,10 @@ def parse_args(args=None):
             help="Set the maximum number of locked shards"),
         optparse.make_option("--additional-env-var", type="string", action="append", default=[],
             help="Passes that environment variable to the tests (--additional-env-var=NAME=VALUE)"),
+        optparse.make_option("--profile", action="store_true",
+            help="Output per-test profile information."),
+        optparse.make_option("--profiler", action="store",
+            help="Output per-test profile information, using the specified profiler."),
     ]))
 
     option_group_definitions.append(("Miscellaneous Options", [
@@ -425,37 +306,79 @@ def parse_args(args=None):
     return option_parser.parse_args(args)
 
 
-def main(argv=None):
+def _set_up_derived_options(port, options):
+    """Sets the options values that depend on other options values."""
+    if not options.child_processes:
+        options.child_processes = os.environ.get("WEBKIT_TEST_CHILD_PROCESSES",
+                                                 str(port.default_child_processes()))
+    if not options.max_locked_shards:
+        options.max_locked_shards = int(os.environ.get("WEBKIT_TEST_MAX_LOCKED_SHARDS",
+                                                       str(port.default_max_locked_shards())))
+
+    if not options.configuration:
+        options.configuration = port.default_configuration()
+
+    if options.pixel_tests is None:
+        options.pixel_tests = port.default_pixel_tests()
+
+    if not options.time_out_ms:
+        options.time_out_ms = str(port.default_timeout_ms())
+
+    options.slow_time_out_ms = str(5 * int(options.time_out_ms))
+
+    if options.additional_platform_directory:
+        additional_platform_directories = []
+        for path in options.additional_platform_directory:
+            additional_platform_directories.append(port.host.filesystem.abspath(path))
+        options.additional_platform_directory = additional_platform_directories
+
+    if not options.http and options.skipped in ('ignore', 'only'):
+        _log.warning("--force/--skipped=%s overrides --no-http." % (options.skipped))
+        options.http = True
+
+    if options.ignore_metrics and (options.new_baseline or options.reset_results):
+        _log.warning("--ignore-metrics has no effect with --new-baselines or with --reset-results")
+
+    if options.new_baseline:
+        options.reset_results = True
+        options.add_platform_exceptions = True
+
+    if options.pixel_test_directories:
+        options.pixel_tests = True
+        varified_dirs = set()
+        pixel_test_directories = options.pixel_test_directories
+        for directory in pixel_test_directories:
+            # FIXME: we should support specifying the directories all the ways we support it for additional
+            # arguments specifying which tests and directories to run. We should also move the logic for that
+            # to Port.
+            filesystem = port.host.filesystem
+            if not filesystem.isdir(filesystem.join(port.layout_tests_dir(), directory)):
+                _log.warning("'%s' was passed to --pixel-test-directories, which doesn't seem to be a directory" % str(directory))
+            else:
+                varified_dirs.add(directory)
+
+        options.pixel_test_directories = list(varified_dirs)
+
+    if options.run_singly:
+        options.verbose = True
+
+
+def run(port, options, args, logging_stream):
+    logger = logging.getLogger()
+    logger.setLevel(logging.DEBUG if options.debug_rwt_logging else logging.INFO)
+
     try:
-        options, args = parse_args(argv)
-        if options.platform and 'test' in options.platform:
-            # It's a bit lame to import mocks into real code, but this allows the user
-            # to run tests against the test platform interactively, which is useful for
-            # debugging test failures.
-            from webkitpy.common.host_mock import MockHost
-            host = MockHost()
-        else:
-            host = Host()
-        port = host.port_factory.get(options.platform, options)
-    except NotImplementedError, e:
-        # FIXME: is this the best way to handle unsupported port names?
-        print >> sys.stderr, str(e)
-        return EXCEPTIONAL_EXIT_STATUS
-    except Exception, e:
-        print >> sys.stderr, '\n%s raised: %s' % (e.__class__.__name__, str(e))
-        traceback.print_exc(file=sys.stderr)
-        raise
+        printer = printing.Printer(port, options, logging_stream, logger=logger)
 
-    logging.getLogger().setLevel(logging.DEBUG if options.debug_rwt_logging else logging.INFO)
-    return run(port, options, args)
+        _set_up_derived_options(port, options)
+        manager = Manager(port, options, printer)
+        printer.print_config(port.results_directory())
 
+        run_details = manager.run(args)
+        _log.debug("Testing completed, Exit status: %d" % run_details.exit_code)
+        return run_details
+    finally:
+        printer.cleanup()
 
-if '__main__' == __name__:
-    try:
-        return_code = main()
-    except BaseException, e:
-        if e.__class__ in (KeyboardInterrupt, TestRunInterruptedException):
-            sys.exit(INTERRUPTED_EXIT_STATUS)
-        sys.exit(EXCEPTIONAL_EXIT_STATUS)
-
-    sys.exit(return_code)
+if __name__ == '__main__':
+    sys.exit(main(sys.argv[1:], sys.stdout, sys.stderr))
